<?php if (!defined('VB_ENTRY')) die('Access denied.');
/*========================================================================*\
|| ###################################################################### ||
|| # vBulletin 5.6.4 Patch Level 1 - Licence Number VBFF772574
|| # ------------------------------------------------------------------ # ||
|| # Copyright 2000-2021 MH Sub I, LLC dba vBulletin. All Rights Reserved.  # ||
|| # This file may not be redistributed in whole or significant part.   # ||
|| # ----------------- VBULLETIN IS NOT FREE SOFTWARE ----------------- # ||
|| # http://www.vbulletin.com | http://www.vbulletin.com/license.html   # ||
|| ###################################################################### ||
\*========================================================================*/

class TwitterLogin_Api_ExternalLogin extends vB_Api_ExternalLogin
{
	protected $disableWhiteList = array(
		// required to show login button template
		'getState',
		'showExternalLoginButton',
		// called by login button template
		'fetchTwitterLinkUrl',
		// called by twitter dialog redirection to log user into forum via their twitter account
		'verifyAuthAndLogin',
	);

	public function __construct()
	{
		parent::__construct();
		$this->library = vB_Library::instance('TwitterLogin:ExternalLogin');
	}

	public function fetchTwitterLinkUrl($origin = "", $redirectTo = "")
	{
		$result = array();

		$loginUrl = $this->library->getLoginPage($origin, $redirectTo);
		if ($loginUrl['success'] AND !empty($loginUrl['url']))
		{
			$result['url'] = $loginUrl['url'];
		}
		else
		{
			$result['error'] = $loginUrl['error'];
		}



		return $result;
	}

	public function unlinkUser()
	{
		$this->library->unlinkCurrentUserFromApp();

		$result = array(
			'success' => true,
		);

		return $result;
	}


	public function getState()
	{
		$userAuth = $this->library->getUserAuthRecord();

		$enabled = $this->library->checkEnabled();

		return array(
			"enabled" => $enabled['enabled'],
			"register_enabled" => $enabled['register_enabled'],
			"external_userid" => !empty($userAuth['external_userid']) ? $userAuth['external_userid'] : '',
		);
	}

	public function forgetRegistrationData($twitterlogin_saved, $url)
	{
		$authRecord = $this->library->getSessionAuthRecord();
		if (!empty($authRecord) AND
			!empty($authRecord['additional_params']['twitterlogin_register_nonce']) AND
			($authRecord['additional_params']['twitterlogin_register_nonce'] == $twitterlogin_saved)
			OR true
		)
		{
			$this->library->forgetRegistrationData();

			$url = $this->addRemoveTwitterloginSavedParamToURL($url);

			return array(
				'success' => true,
				'url' => $url,
			);
		}
		return array(
			'success' => false,
			'error' => "twitterlogin_saved not found",
		);
	}

	private function addRemoveTwitterloginSavedParamToURL($url, $twitterlogin_saved = null)
	{
		$url_parts = parse_url($url);
		$queryString = $url_parts['query'] ?? '';
		$query = [];
		parse_str($queryString, $query);
		unset($query['twitterlogin_saved']);
		if (!empty($twitterlogin_saved))
		{
			$query['twitterlogin_saved'] = $twitterlogin_saved;
		}
		foreach ($query AS $__key => $__val)
		{
			$query[$__key] = $__key . "=" . $__val;
		}
		$url = $url_parts['scheme'] . "://" . $url_parts['host'] . $url_parts['path'] . '?' . implode('&', $query);

		return $url;
	}

	public function getRegistrationData($twitterlogin_saved)
	{
		/*
			The nonce check is mostly meant to help ensure that the user requested that their twitter account
			be added during this specific session/flow, not at a previous point (e.g. a different person
			on a shared computer).
			This might need more refinement, however.
		 */
		$authRecord = $this->library->getSessionAuthRecord();
		if (!empty($authRecord) AND
			!empty($authRecord['additional_params']['twitterlogin_register_nonce']) AND
			($authRecord['additional_params']['twitterlogin_register_nonce'] == $twitterlogin_saved)
		)
		{
			$result = array(
				'found'           => true,
				'username'        => $authRecord['additional_params']['screen_name'],
				'email'           => $authRecord['additional_params']['email'],
				'external_userid' =>  $authRecord['additional_params']['external_userid'],
				'return_to_url'   =>  $authRecord['additional_params']['return_to_url'],
				'profile_image_url_https' => $authRecord['additional_params']['profile_image_url_https'],
				'url'             => "https://twitter.com/" . $authRecord['additional_params']['screen_name'],
			);
		}
		else
		{
			$result = array(
				'found'           => false,
			);
		}

		return $result;
	}

	public function isSiteUrl($url)
	{
		$vurl = vB::getUrlLoader();
		if (!empty($url) AND $vurl->isSiteUrl($url))
		{
			return true;
		}

		return false;
	}

	public function verifyAuthForvBRegistration($libid, $oauth_token, $oauth_verifier, $nonce, $denied = null, $resume_url = null)
	{
		// If available, link back to the registration page that contains the urlpath param.
		if (!$this->isSiteUrl($resume_url))
		{
			// only allow internal redirects
			$resume_url = null;
		}
		$resume_url = $resume_url ?? vB5_Route::buildUrl('register|fullurl');

		if (!empty($denied))
		{
			return [
				'error' => ['twitterlogin_error_need_app_auth__register', $resume_url],
			];
		}

		$generic_error_phrase = 'twitterlogin_error_invalid_token';
		if (!$this->checkCallbackParams($libid, $oauth_token, $oauth_verifier, $nonce))
		{
			return [
				'error' => $generic_error_phrase,
			];
		}

		/*
			If we got this far, assume everything's green and the user has authenticated us.
			Convert the request token into an access token, get the twitter userid, save it to
			userauth.external_userid, & this user can start using twitter to log in!
			probably.
		 */
		$params = [
			'oauth_token' => $oauth_token,
			'oauth_verifier' => $oauth_verifier,
		];


		/*
			If we successfully fetched twitter data & stored it, we need to pass a special
			twitterlogin_saved query param over to the register page so that the
			twitterlogin_registeruserbutton template can re-fetch that data.
			We append it to resume_url in order to preserve the urlpath queryparam that's
			used post-registration to redirect the user back to the page they were on before
			they decided to register.
		 */
		$check = $this->library->fetchAndStoreTwitterInfoForRegistration($params);
		if (!empty($check['twitterlogin_saved']))
		{
			$check['resume_url'] = $this->addRemoveTwitterloginSavedParamToURL($resume_url, $check['twitterlogin_saved']);
		}

		return $check;
	}

	public function verifyAuthAndLogin($libid, $oauth_token, $oauth_verifier, $nonce, $denied = null, $resume_url = null)
	{
		if (!empty($denied))
		{
			return [
				'error' => 'twitterlogin_error_need_app_auth',
			];
		}

		$generic_error_phrase = 'twitterlogin_error_invalid_token';
		if (!$this->checkCallbackParams($libid, $oauth_token, $oauth_verifier, $nonce))
		{
			return [
				'error' => $generic_error_phrase,
			];
		}

		/*
			If we got this far, assume everything's green and the user has authenticated us.
			Convert the request token into an access token, get the twitter userid, save it to
			userauth.external_userid, & this user can start using twitter to log in!
			probably.
		 */
		$params = [
			'oauth_token' => $oauth_token,
			'oauth_verifier' => $oauth_verifier,
		];
		$check = $this->library->getLinkedVBUseridFromRequestTokens($params);
		if ($check['success'] AND !empty($check['userid']))
		{
			// log in using userid
			$result = $this->library->loginUser($check['userid']);
			// Note, page controller handles redirecting to $resume_url after
			// setting login cookies so we don't have to send it back here.
			return ($check + $result);
		}
		else if ($check['error'] == 'no_oauth_user_found' AND
			!empty($check['external_userid']) AND
			!empty($check['userauth'])
		)
		{
			// store this for auto-linking and append a link to registration page.
			$check2 = $this->library->fetchAndStoreTwitterInfoForRegistration([], $check['userauth']);

			if (!empty($check2['twitterlogin_saved']))
			{
				$frontendUrl = vB::getDatastore()->getOption('frontendurl');
				$registrationUrl = $frontendUrl . "/register";
				if ($this->isSiteUrl($resume_url))
				{
					$registrationUrl .= "?urlpath=" . base64_encode($resume_url);
				}
				$registrationUrl = $this->addRemoveTwitterloginSavedParamToURL($registrationUrl, $check2['twitterlogin_saved']);
				$check['error_redirect'] = $registrationUrl;
			}
		}

		return $check;
	}

	private function checkCallbackParams($libid, $oauth_token, $oauth_verifier, $nonce)
	{
		/*
			This was originally to mitigate brute force attacks against the userauth table
			when we used to use that instead of sessionauth.
			Limit brute force attacks vs. any existing records by verifying that
			additional_params.verify_token_nonce matches nonce
		 */

		$authRecord = $this->library->getSessionAuthRecord($oauth_token);
		if (empty($authRecord))
		{
			/*
				We have no idea what happened here. Either callback URL is incorrect or the
				initial get request token failed badly and we shouldn't ever have gotten to
				the auth step, but somehow did, or something else entirely.
				We don't know how to recover from this
			 */
			return false;
		}

		if ($authRecord['loginlibraryid'] != $libid OR
			// todo, is there a need to one-way hash the nonce check?
			empty($authRecord['additional_params']['verify_token_nonce']) OR
			$authRecord['additional_params']['verify_token_nonce'] != $nonce
		)
		{
			/*
				Again, not sure how we got here, but they supplied us with the wrong
				library or userid and the token does NOT match up.
				Suspect foul play, and generic error up.
			 */
			return false;
		}


		return true;
	}

	public function verifyAuthAndLinkUser($libid, $oauth_token, $oauth_verifier, $nonce, $denied = null, $resume_url = null)
	{
		$generic_error_phrase = 'twitterlogin_error_invalid_token';

		$userAuth = $this->library->getUserAuthRecord();
		if (!empty($userAuth['external_userid']))
		{
			/*
				no bueno, someone might be trying to brute force our tokens or
				overwrite an existing token	in an unsupported way.
				todo: wait, how *do* we refresh the token? Unlink first?
			 */
			return [
				'error' => $generic_error_phrase,
			];
		}

		// If the user denied this twitter app's access to their account, we may have a
		// valid sessionauth (& thus checkCallbackParams() passes) but still get a denied
		// param.
		if (!empty($denied))
		{
			return [
				'error' => 'twitterlogin_error_need_app_auth__connect',
			];
		}

		if (!$this->checkCallbackParams($libid, $oauth_token, $oauth_verifier, $nonce))
		{
			return [
				'error' => $generic_error_phrase,
			];
		}

		/*
			If we got this far, assume everything's green and the user has authenticated us.
			Convert the request token into an access token, get the twitter userid, save it to
			userauth.external_userid, & this user can start using twitter to log in!
			probably.
		 */
		$params = [
			'oauth_token' => $oauth_token,
			'oauth_verifier' => $oauth_verifier,
		];
		$linkResult = $this->library->linkCurrentUserWithApp($params);

		return $linkResult;
	}
}

/*=========================================================================*\
|| #######################################################################
|| # Downloaded: 21:58, Tue Jul 6th 2021
|| # CVS: $RCSfile$ - $Revision: 105377 $
|| #######################################################################
\*=========================================================================*/
